@@ -26,7 +26,7 @@ from scipy.special import inv_boxcox
 #%%
 def hts(y, h = 1, nodes = [[2]], method='OLS', freq = 'D', transform = None, include_history = True, cap = None, capF = None, changepoints = None, \
         n_changepoints = 25, yearly_seasonality = 'auto', weekly_seasonality = 'auto', holidays = None, seasonality_prior_scale = 10.0, \
-        holidays_prior_scale = 10.0, changepoint_prior_scale = 0.05, mcmc_samples = 0, interval_width = 0.80, uncertainty_samples = 0):
+        holidays_prior_scale = 10.0, changepoint_prior_scale = 0.05, mcmc_samples = 0, interval_width = 0.80, uncertainty_samples = 0, skipFitting = False):
     '''
     Parameters
     ----------------
@@ -149,6 +149,8 @@ def hts(y, h = 1, nodes = [[2]], method='OLS', freq = 'D', transform = None, inc
     if isinstance(capF, pd.DataFrame):
         if len(capF.columns) != len(y.columns)-1:
             sys.exit("If capF is a DataFrame, it should have a number of columns equal to the input Dataframe - 1")
+    if cap is not None and method not in ["BU","FP","AHP","PHA"]:
+        print("Consider using BU, FP, AHP, or PHA.  The other methods can create negatives which would cause problems for the log() function")
     ##
     # Transform Variables
     ##
@@ -168,6 +170,8 @@ def hts(y, h = 1, nodes = [[2]], method='OLS', freq = 'D', transform = None, inc
                 for column in range(len(y.columns.tolist())-1):
                     y.iloc[:,column+1] = boxcox(y.iloc[:, column+1], lmbda = 0)
                     boxcoxT[column] = 0
+        else:
+            print("Nothing will be transformed because the input was not = to 'BoxCox'")
     else:
         boxcoxT = None
     ##
@@ -177,7 +181,7 @@ def hts(y, h = 1, nodes = [[2]], method='OLS', freq = 'D', transform = None, inc
         ##
         # Run all of the Methods and let 3 fold CV chose which is best for you
         ##
-        methodList = ['OLS','FP','PHA','AHP','BU']
+        methodList = ['WLSV','WLSS','OLS','FP','PHA','AHP','BU']
         sumMat = SummingMat(nodes)
         tscv = TimeSeriesSplit(n_splits=3)
         MASE1 = []
@@ -185,51 +189,77 @@ def hts(y, h = 1, nodes = [[2]], method='OLS', freq = 'D', transform = None, inc
         MASE3 = []
         MASE4 = []
         MASE5 = []
+        MASE6 = []
+        MASE7 = []
         ##
         # Split into train and test, using time series split, and predict the test set
         ##
+        y1 = y.copy()
+        if boxcoxT is not None:
+                for column in range(len(y.columns.tolist())-1):
+                    y1.iloc[:,column+1] = inv_boxcox(y1.iloc[:, column+1], boxcoxT[column])
+                    
         for trainIndex, testIndex in tscv.split(y.iloc[:,0]):
             ynew1 = fitForecast(y.iloc[trainIndex, :], len(testIndex), sumMat, nodes, methodList[0], freq, include_history, cap, capF, changepoints, n_changepoints, \
                                 yearly_seasonality, weekly_seasonality, holidays, seasonality_prior_scale, holidays_prior_scale,\
-                                changepoint_prior_scale, mcmc_samples, interval_width, uncertainty_samples, boxcoxT)
+                                changepoint_prior_scale, mcmc_samples, interval_width, uncertainty_samples, boxcoxT, skipFitting)
             ynew2 = fitForecast(y.iloc[trainIndex, :], len(testIndex), sumMat, nodes, methodList[1], freq, include_history, cap, capF, changepoints, n_changepoints, \
                                 yearly_seasonality, weekly_seasonality, holidays, seasonality_prior_scale, holidays_prior_scale,\
-                                changepoint_prior_scale, mcmc_samples, interval_width, uncertainty_samples, boxcoxT)
+                                changepoint_prior_scale, mcmc_samples, interval_width, uncertainty_samples, boxcoxT, skipFitting)
             ynew3 = fitForecast(y.iloc[trainIndex, :], len(testIndex), sumMat, nodes, methodList[2], freq, include_history, cap, capF, changepoints, n_changepoints, \
                                 yearly_seasonality, weekly_seasonality, holidays, seasonality_prior_scale, holidays_prior_scale,\
-                                changepoint_prior_scale, mcmc_samples, interval_width, uncertainty_samples, boxcoxT)
+                                changepoint_prior_scale, mcmc_samples, interval_width, uncertainty_samples, boxcoxT, skipFitting)
             ynew4 = fitForecast(y.iloc[trainIndex, :], len(testIndex), sumMat, nodes, methodList[3], freq, include_history, cap, capF, changepoints, n_changepoints, \
                                 yearly_seasonality, weekly_seasonality, holidays, seasonality_prior_scale, holidays_prior_scale,\
-                                changepoint_prior_scale, mcmc_samples, interval_width, uncertainty_samples, boxcoxT)
+                                changepoint_prior_scale, mcmc_samples, interval_width, uncertainty_samples, boxcoxT, skipFitting)
             ynew5 = fitForecast(y.iloc[trainIndex, :], len(testIndex), sumMat, nodes, methodList[4], freq, include_history, cap, capF, changepoints, n_changepoints, \
                                 yearly_seasonality, weekly_seasonality, holidays, seasonality_prior_scale, holidays_prior_scale,\
-                                changepoint_prior_scale, mcmc_samples, interval_width, uncertainty_samples, boxcoxT)
+                                changepoint_prior_scale, mcmc_samples, interval_width, uncertainty_samples, boxcoxT, skipFitting)
+            ynew6 = fitForecast(y.iloc[trainIndex, :], len(testIndex), sumMat, nodes, methodList[5], freq, include_history, cap, capF, changepoints, n_changepoints, \
+                                yearly_seasonality, weekly_seasonality, holidays, seasonality_prior_scale, holidays_prior_scale,\
+                                changepoint_prior_scale, mcmc_samples, interval_width, uncertainty_samples, boxcoxT, skipFitting)
+            ynew7 = fitForecast(y.iloc[trainIndex, :], len(testIndex), sumMat, nodes, methodList[6], freq, include_history, cap, capF, changepoints, n_changepoints, \
+                                yearly_seasonality, weekly_seasonality, holidays, seasonality_prior_scale, holidays_prior_scale,\
+                                changepoint_prior_scale, mcmc_samples, interval_width, uncertainty_samples, boxcoxT, skipFitting)
+                    
             for key in ynew1.keys():
-                MASE1.append(abs(ynew1[key].yhat[-len(testIndex):].values - y.iloc[testIndex, key+1].values)/np.mean(abs(y.iloc[trainIndex[1:], key + 1].values - y.iloc[trainIndex[:-1], key + 1].values)))
-                MASE2.append(abs(ynew2[key].yhat[-len(testIndex):].values - y.iloc[testIndex, key+1].values)/np.mean(abs(y.iloc[trainIndex[1:], key + 1].values - y.iloc[trainIndex[:-1], key + 1].values)))
-                MASE3.append(abs(ynew3[key].yhat[-len(testIndex):].values - y.iloc[testIndex, key+1].values)/np.mean(abs(y.iloc[trainIndex[1:], key + 1].values - y.iloc[trainIndex[:-1], key + 1].values)))
-                MASE4.append(abs(ynew4[key].yhat[-len(testIndex):].values - y.iloc[testIndex, key+1].values)/np.mean(abs(y.iloc[trainIndex[1:], key + 1].values - y.iloc[trainIndex[:-1], key + 1].values)))
-                MASE5.append(abs(ynew5[key].yhat[-len(testIndex):].values - y.iloc[testIndex, key+1].values)/np.mean(abs(y.iloc[trainIndex[1:], key + 1].values - y.iloc[trainIndex[:-1], key + 1].values)))
+                MASE1.append(abs(ynew1[key].yhat[-len(testIndex):].values - y1.iloc[testIndex, key+1].values)/np.mean(abs(y1.iloc[trainIndex[1:], key + 1].values - y1.iloc[trainIndex[:-1], key + 1].values)))
+                MASE2.append(abs(ynew2[key].yhat[-len(testIndex):].values - y1.iloc[testIndex, key+1].values)/np.mean(abs(y1.iloc[trainIndex[1:], key + 1].values - y1.iloc[trainIndex[:-1], key + 1].values)))
+                MASE3.append(abs(ynew3[key].yhat[-len(testIndex):].values - y1.iloc[testIndex, key+1].values)/np.mean(abs(y1.iloc[trainIndex[1:], key + 1].values - y1.iloc[trainIndex[:-1], key + 1].values)))
+                MASE4.append(abs(ynew4[key].yhat[-len(testIndex):].values - y1.iloc[testIndex, key+1].values)/np.mean(abs(y1.iloc[trainIndex[1:], key + 1].values - y1.iloc[trainIndex[:-1], key + 1].values)))
+                MASE5.append(abs(ynew5[key].yhat[-len(testIndex):].values - y1.iloc[testIndex, key+1].values)/np.mean(abs(y1.iloc[trainIndex[1:], key + 1].values - y1.iloc[trainIndex[:-1], key + 1].values)))
+                MASE6.append(abs(ynew6[key].yhat[-len(testIndex):].values - y1.iloc[testIndex, key+1].values)/np.mean(abs(y1.iloc[trainIndex[1:], key + 1].values - y1.iloc[trainIndex[:-1], key + 1].values)))
+                MASE7.append(abs(ynew7[key].yhat[-len(testIndex):].values - y1.iloc[testIndex, key+1].values)/np.mean(abs(y1.iloc[trainIndex[1:], key + 1].values - y1.iloc[trainIndex[:-1], key + 1].values)))
         ##
         # If the method has the minimum Average MASE, use it on all of the data
         ##
-        choices = [np.mean(MASE1), np.mean(MASE2), np.mean(MASE3), np.mean(MASE4), np.mean(MASE5)]
+        choices = [np.mean(MASE1), np.mean(MASE2), np.mean(MASE3), np.mean(MASE4), np.mean(MASE5), np.mean(MASE6), np.mean(MASE7)]
         choice = methodList[choices.index(min(choices))]
         ynew = fitForecast(y, h, sumMat, nodes, choice, freq, include_history, cap, capF, changepoints, n_changepoints, \
                            yearly_seasonality, weekly_seasonality, holidays, seasonality_prior_scale, holidays_prior_scale,\
-                           changepoint_prior_scale, mcmc_samples, interval_width, uncertainty_samples, boxcoxT)
+                           changepoint_prior_scale, mcmc_samples, interval_width, uncertainty_samples, boxcoxT, skipFitting)
         print(choice)
     
     else:    
         sumMat = SummingMat(nodes)
         ynew = fitForecast(y, h, sumMat, nodes, method, freq, include_history, cap, capF, changepoints, n_changepoints, \
                            yearly_seasonality, weekly_seasonality, holidays, seasonality_prior_scale, holidays_prior_scale,\
-                           changepoint_prior_scale, mcmc_samples, interval_width, uncertainty_samples, boxcoxT)
+                           changepoint_prior_scale, mcmc_samples, interval_width, uncertainty_samples, boxcoxT, skipFitting)
         
     if transform is not None:
         if transform == 'BoxCox':
             for column in range(len(y.columns.tolist())-1):
                 y.iloc[:,column+1] = inv_boxcox(y.iloc[:, column+1], boxcoxT[column])
+    ##
+    # Rename keys so that dictionary can be easily understood
+    ##
+    i = -2
+    for column in y:
+        i += 1   
+        if i == -1:
+            continue
+        else:
+            ynew[column] = ynew.pop(i)
     
     return ynew
 
@@ -369,12 +399,12 @@ def orderHier(data, col1 = 1, col2 = None, col3 = None, col4 = None, rmZeros = F
                                                == uniqueList[orderList.index(2)][ind]) & (placeholder2.index.get_level_values(2) == uniqueList[orderList.index(3)][cnt]) & (placeholder2.index.get_level_values(3) == uniqueList[orderList.index(4)][pos])]).to_frame()
     
     #Creating total DataFrame
-    allDataframes['total'] = (data.groupby([timeInterval])[numCol].sum()).to_frame()
+    allDataframes['Total'] = (data.groupby([timeInterval])[numCol].sum()).to_frame()
 
     #Take the sessions column from all those dataframes and put them into y
     y = pd.DataFrame(data = data[timeInterval].unique(), index = range(len(data[timeInterval].unique())), columns = ['time'])
-    y = pd.merge(y, allDataframes['total'], left_on = 'time', right_index = True)
-    y.rename(columns = {numCol:'total'}, inplace = True)
+    y = pd.merge(y, allDataframes['Total'], left_on = 'time', right_index = True)
+    y.rename(columns = {numCol:'Total'}, inplace = True)
     
     for i in range(lengthList[orderList.index(1)]):
         y = pd.merge(y, allDataframes[uniqueList[orderList.index(1)][i]+'1'], how = 'left', left_on = 'time', right_index = True)
